import java.util.Map;
import java.util.TreeMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Comparator;

public class MapDemo {
    
    public static void main(String[] args) {

       Map<Person, Double> people = new TreeMap<>(
        new PersonYearOfBirthDescendingComparator());
       
       Person honza = new Person("Honza", 2000);
       Person petr = new Person("Petr", 1980);
       Person ales = new Person("Aleš", 1960);
       Person libor = new Person("Libor", 1980);
 
       people.put(honza, 12000.0);
       people.put(petr, 3000.0);
       people.put(ales, 300000.0);
       people.put(libor, 1000.0);

       for(Map.Entry<Person, Double> p: people.entrySet()) {
           Person person = p.getKey();
           double salary = p.getValue();
           System.out.println(person.getName() + " born "
            +person.getYearOfBirth() 
            +" has salary "+salary);
       }
      
       //System.out.println("Honza has salary="
       // +people.get(new Person("Honza")));
    }
}
class Person implements Comparable<Person> {
    
    private String name;
    private int yearOfBirth;
    public Person(String name, int yearOfBirth) { 
        this.name = name; 
        this.yearOfBirth = yearOfBirth;
    }
    public String getName() { return name; }
    public void setName(String name) { this.name = name; } 
    
    public int getYearOfBirth() { return yearOfBirth; }
    public void setYearOfBirth(int yearOfBirth) { 
        this.yearOfBirth = yearOfBirth; 
    } 
    
    public boolean equals(Object o) {
        if(o instanceof Person) {
            Person p = (Person)o;
            return this.name.equals(p.name);
        } else return false;
    }
    
    public int hashCode() {
        return name.hashCode();
    }
    
    public int compareTo(Person p) {
        return this.name.compareTo(p.name);
    }
}
class PersonYearOfBirthComparator implements Comparator<Person> {
    public int compare(Person p1, Person p2) {
        int diff = p1.getYearOfBirth() - p2.getYearOfBirth();
        if(diff == 0) {
            return p1.compareTo(p2);
        }
        return diff;
    }
}
class PersonYearOfBirthDescendingComparator extends PersonYearOfBirthComparator {
    public int compare(Person p1, Person p2) {
        return -super.compare(p1, p2);
    }
}